#!/usr/bin/env python3
from netCDF4 import Dataset
import numpy as np
from array import *
#
# Program to create morel f/Q input netcdf file for brdf correction
# R. Healy 7/23/2015

# create a file (Dataset object, also the root group).

N_W = 7
N_S = 6
N_C = 6
N_N = 17
N_A = 13

wavetab = array('f',[412.5,442.5,490.0,510.0,560.0,620.0,660.0]) # N_W
solztab = array('f',[0.,15.,30.,45.,60.,75.,])                    # N_S
chltab  = array('f',[0.03,0.1,0.3,1.0,3.0,10.0,])                 # N_C
senztab = array('f',[1.078,3.411,6.289,9.278,12.300,15.330,18.370,21.410,24.450,
           27.500,30.540,33.590,36.640,39.690,42.730,45.780,48.830,])    # N_N
phitab  = array('f',[0.,15.,30.,45.,60.,75.,90.,105.,120.,135.,150.,165.,180.,])   # N_A

datain = np.empty([N_W,N_S,N_C,N_N,N_A], dtype=float)

d = np.loadtxt('morel_fq.dat',delimiter=' ')

print ("shape=" + str(d.shape))
nx,ny = d.shape
print ("nx,ny=" + str(nx) + str(ny))
b = d[0,1]
print(str(b))

rootgrp = Dataset('test.nc', 'w', format='NETCDF4')
print (rootgrp.file_format)
rootgrp.close()

# create some groups.
rootgrp = Dataset('morel_fq.nc', 'w')
#obsgrp = rootgrp.createGroup('observation_data')
print (rootgrp.groups)

# walk the group tree using a Python generator.
def walktree(top):
    values = top.groups.values()
    yield values
    for value in top.groups.values():
        for children in walktree(value):
            yield  children

print (rootgrp.path + " <*" + str(rootgrp) + " *>" )

for children in walktree(rootgrp):
     for child in children:
         print (child.path + "***" + str(child))

# dimensions.
rootgrp.createDimension('n_phi',  13)
rootgrp.createDimension('n_senz', 17)
rootgrp.createDimension('n_chl',   6)
rootgrp.createDimension('n_solz',  6)
rootgrp.createDimension('n_wave',  7)

print (str(rootgrp.dimensions))

for dimname, dimobj in rootgrp.dimensions.items():
    print (dimname + " " + str(len(dimobj)) + " " +  str(dimobj.isunlimited()))

# variables.
phi = rootgrp.createVariable('phi','f4',('n_phi',))
senz = rootgrp.createVariable('senz','f4',('n_senz',))
chl = rootgrp.createVariable('chl','f4',('n_chl',))
solz = rootgrp.createVariable('solz','f4',('n_solz',))
wave = rootgrp.createVariable('wave','f4',('n_wave',))

foq = rootgrp.createVariable('foq','f4',('n_wave','n_solz','n_chl','n_senz','n_phi',)) #,least_significant_digit=3)

# attributes.
import time


rootgrp.description = 'F/Q'
rootgrp.history = 'Created ' + time.ctime(time.time())
rootgrp.source = 'cre_morel_ncfile.py'
rootgrp.author = 'Rick Healy richard.healy@nasa.gov'
wave.units = 'um'
solz.units = 'degrees'
senz.units = 'degrees'
chl.units = 'mg m^-3'
phi.units = 'degrees'

for name in rootgrp.ncattrs():
    print ('Global attr' + name + '=' + getattr(rootgrp,name))

print (rootgrp.__dict__)

print (rootgrp.variables)

print(str(wavetab))

i = 0
for wav in wavetab:
    wave[i] = wav
    i += 1

i = 0
for sol in solztab:
    solz[i] = sol
    i += 1

i = 0
for sen in senztab:
    senz[i] = sen
    i += 1

i = 0
for ph in phitab:
    phi[i] = ph
    print('phi{0}={1}'.format(i,phi[i]))
    i += 1

i = 0
for ch in chltab:
    chl[i] = ch
    i += 1


print ('phi =\n' + " " + str(phi[:]))
print ('chl =\n' + " " + str(chl[:]))
print ('senz =\n' + " " + str(senz[:]))
print ('solz =\n' + " " + str(solz[:]))
print ('wave =\n' + " " + str(wave[:]))

# append along two unlimited dimensions by assigning to slice.
nphi = len(rootgrp.dimensions['n_phi'])
nchl = len(rootgrp.dimensions['n_chl'])
print ('foq shape before adding data = ' + " " + str(foq.shape))
for i in range(0,N_W):
    for j in range(0,N_S):
        for k in range(0,N_C):
            for l in range(0,N_N):
                for m in range(0,N_A):
#                    print(str(i*N_S*N_C*N_N+j*N_C*N_N+k*N_N+l))
                    foq[i,j,k,l,m] = d[i*N_S*N_C*N_N+j*N_C*N_N+k*N_N+l,m]

print ('foq shape after adding data = ' + " " + str(foq.shape))

rootgrp.close()
